/******************************************************************************
* Copyright (C) 2008 - Ikaro Games   www.ikarogames.com                       *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; either version 2              *
* of the License, or (at your option) any later version.                      *
*                                                                             *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with this program; if not, write to the Free Software                 *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                             *
*                                                                             *
******************************************************************************/


#ifndef __CSimulationManager_H__
#define __CSimulationManager_H__


#include <string>
#include <vector>

#include "entity/CFootballPlayer.h"
#include "entity/CReferee.h"
#include "entity/CBall.h"
#include "../utils/CTimer.h"
#include "entity/CTeam.h"
#include "CSimulationWorld.h"


class CSimulationManager
{
public:
    CSimulationManager();
    ~CSimulationManager();

    void update();

    void startMatch();
    CSimulationWorld* getSimulationWorld();
    CReferee* getReferee();
    CBall* getBall();
    CTeam* getHomeTeam();
    CTeam* getAwayTeam();
    void changePlayersSide();

    //Information Methods
    std::string* getHomeTeamName();
    std::string* getAwayTeamName();
    btVector3 getBallPosition() const;


    // Soccer Commands
    void dash(CFootballPlayer *player, btVector3 power);
    void move(CFootballPlayer *player, int x, int z);
    void kick(CFootballPlayer *player, btVector3 power);
    void turn(CFootballPlayer *player, int moment);

private:
    CTimer *m_logicTimer;
    CTimer *m_physicsTimer;
    CTeam *m_homeTeam;
    CTeam *m_awayTeam;
    CReferee *m_referee;
    CBall *m_ball;
    CSimulationWorld *m_simWorld;

    void truncateVector(btVector3 *v, double max);
};

#endif // __CSimulationManager_H__
